package org.jutils.collection;
import java.util.*;

public class Associator {
   /** Contains the keys and value vectors. */
   // private Hashtable relationalData_ = new Hashtable();
   private Map relationalData_ = new Hashtable();

   /** The name of the class used to factor Set instances with. */
   private Class setClass_ = HashSet.class;

   /** Defines wether or not this asociator works bidirectional. */
   private boolean bidirectional_ = false;

   private String listOpenBracket_ = "{";
   private String listCloseBracket_ = "}";
   private String listSeparator_ = ",";



   /**
    * OneToManyRelation constructor comment.
    */
   public Associator() {
    super();
   }



   /**
    * OneToManyRelation constructor comment.
    */
   public Associator(boolean bidirectional) {
    super();
    bidirectional_ = bidirectional;
   }



/**
* Defines a new association between this two values.
* <br>Creation date: (27.07.2000 12:51:05)<br>
* @author: M.Winkler, doubleSlash GmbH
*
* @param key java.lang.Object
* @param newValue java.lang.Object
*/
public void associate(Object key, Object newValue) {
   pAssociate(key, newValue);
   if (bidirectional_)
      pAssociate(newValue, key);
}



/**
* clear method comment.
*/
public void clear() {

   // remove all elements (to avoid loitering objects)
   Object keys[] = relationalData_.keySet().toArray();
   for (int i = 0; i < keys.length; i++)
      pRemove(keys[i]);

   // clear map
   relationalData_.clear();
}



/**
    * Code to perform when this object is garbage collected.
    *
    * Any exception thrown by a finalize method causes the finalization to
    * halt. But otherwise, it is ignored.
    */
protected void finalize() throws Throwable {
   // Insert code to finalize the receiver here.
   // This implementation simply forwards the message to super.  You may replace or supplement this.
   super.finalize();
}



   /**
   * @author Michael Winkler, doubleSlash GmbH
   *
   * @return java.lang.Object[]
   */
   public Object[] getKeysArray() {
    Object[] returnValue = null;
    List keysList = keysList();
    returnValue = keysList.toArray();
    return returnValue;
   }



/**
    * Tests the class.
    * @param args an array of command-line arguments
    */
public static void main(java.lang.String[] args) {
   System.out.println("Testing Associator with default set class ...");
   int errorCount = 0;
   System.out.println("initializing ...");
   Associator a = new Associator(false);
   int initialCount = 10000;
   long startTime = System.currentTimeMillis();

   // putting "initialCount" elements intop the associator
   for (int index = 0; index < initialCount; index++) {
      Integer key = new Integer(index);
      Integer value = new Integer(index + initialCount);

      // associationg unidirectional key with value
      a.associate(key, value);
   }
   long timeUsed = System.currentTimeMillis() - startTime;
   System.out.println("Needed " + timeUsed + " ms to create " + initialCount * 2 + " entries.");
   double timePerOp = (double) timeUsed / (initialCount * 2);
   System.out.println("That is " + timePerOp + " ms per operation.");
   System.out.println();
   System.out.println("Checking consistency ...");
   startTime = System.currentTimeMillis();

   // search these associations
   for (int index = 0; index < initialCount; index++) {
      Integer key = new Integer(index);
      Integer value = new Integer(index + initialCount);

      // retrieving associations
      Set associationsSet = a.getAssociations(key);
      if (!associationsSet.contains(value)) {
         System.err.println("Error! association not found!");
         errorCount++;
      }

      // check removal
      a.remove(key, value);
      associationsSet = a.getAssociations(key);
      if (associationsSet.size() > 1) {
         System.err.println("Error! count does not match aftrer removing assotiation for " + key + "!");
         System.err.println("Associations : " + associationsSet.toString());
         errorCount++;
      }
   }

   timeUsed = System.currentTimeMillis() - startTime;
   System.out.println("Needed " + timeUsed + " ms to check " + initialCount + " entries.");
   System.out.println();

   // check with sorted TreeSet
   // ---------------------------------------------
   System.out.println("Testing Associator with default set class ...");
   a.clear();
   try {
      a.setSetClass(TreeSet.class);
   } catch (Exception e) {
      System.out.println("ERROR! cannot set SetClass!");
      errorCount++;
   }

   // putting "initialCount" elements intop the associator
   for (int index = 0; index < initialCount; index++) {
      Integer key = new Integer(index);
      Integer value = new Integer(index + initialCount);

      // associationg unidirectional key with value
      a.associate(key, value);
   }
   timeUsed = System.currentTimeMillis() - startTime;
   System.out.println("Needed " + timeUsed + " ms to create " + initialCount * 2 + " entries.");
   timePerOp = (double) timeUsed / (initialCount * 2);
   System.out.println("That is " + timePerOp + " ms per operation.");
   System.out.println();
   System.out.println("Checking consistency ...");
   startTime = System.currentTimeMillis();

   // search these associations
   for (int index = 0; index < initialCount; index++) {
      Integer key = new Integer(index);
      Integer value = new Integer(index + initialCount);

      // retrieving associations
      Set associationsSet = a.getAssociations(key);
      if (!associationsSet.contains(value)) {
         System.err.println("Error! association not found!");
         errorCount++;
      }

      // check removal
      a.remove(key, value);
      associationsSet = a.getAssociations(key);
      if (associationsSet.size() > 1) {
         System.err.println("Error! count does not match aftrer removing assotiation for " + key + "!");
         System.err.println("Associations : " + associationsSet.toString());
         errorCount++;
      }
   }

   // stats
   System.out.println();
   System.out.println("" + errorCount + " errors found.");
}



/**
    * Does the association, one-way.
    * <br>Creation date: (27.07.2000 13:06:00)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @param key java.lang.Object
    * @param value java.lang.Object
    */
private void pAssociate(Object key, Object value) {

   // retrieve existing vector
   Set s = (Set) relationalData_.get(key);
   if (s == null) {
      s = pNewSet();
      relationalData_.put(key, s);
   }
   s.add(value);
}



/**
    * Removes the key unidirectional.
    * <br>Creation date: (27.07.2000 12:56:41)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return boolean
    * @param key java.lang.Object
    * @param value java.lang.Object
    */
private boolean pRemove(Object key, Object value) {
   boolean returnValue = false;

   // First, find key by retrieving vector
   Set s = (Set) relationalData_.get(key);
   if (s != null)
      returnValue = s.remove(value);

   return returnValue;
}



/**
   * <br>Creation date: (27.07.2000 12:56:41)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return boolean
   * @param key java.lang.Object
   * @param value java.lang.Object
   */
public boolean remove(Object key, Object value) {
   boolean returnValue = false;
   returnValue = pRemove(key, value);
   if (bidirectional_)
      returnValue = returnValue || pRemove(value, key);
   return returnValue;
}



   /**
   * setListChars method comment.
   */
   public void setListChars(java.lang.String openBracket, java.lang.String separator, java.lang.String closeBracket) {
    listOpenBracket_ = openBracket;
    listSeparator_ = separator;
    listCloseBracket_ = closeBracket;
   }



   /**
    * Returns a String that represents the value of this object.
    * @return a string representation of the receiver
    */
   public String toString() {
    return toString(listOpenBracket_, listSeparator_, listCloseBracket_);
   }



   /**
   * toString method comment.
   */
   public java.lang.String toString(java.lang.String openBracket, java.lang.String separator, java.lang.String closeBracket) {
    StringBuffer returnValue = new StringBuffer();
    if (openBracket != null)
      returnValue.append(openBracket);
    Iterator keys = relationalData_.keySet().iterator();
    boolean alreadyAddedSomething = false;
    while (keys.hasNext()) {
      if (alreadyAddedSomething)
       returnValue.append(separator);
      returnValue.append('(');
      Object key = keys.next();
      returnValue.append(key);
      if (bidirectional_)
       returnValue.append('<');
      returnValue.append("=>");
      Object associations = relationalData_.get(key);
      returnValue.append(associations);
      returnValue.append(')');
      alreadyAddedSomething = true;
    }
    if (closeBracket != null)
      returnValue.append(closeBracket);
    return returnValue.toString();
   }



/**
* @author Michael Winkler, doubleSlash GmbH
*
* @return java.lang.Object[]
*/
public List keysList() {
   List returnValue = null;
   // Enumeration keysEnumeration = relationalData_.keys();
   Set keysSet = relationalData_.keySet();
   returnValue = new LinkedList(keysSet);
   return returnValue;
}



/**
* Associates the passed key with all the passed values.
* <br>Creation date: (27.07.2000 12:51:05)<br>
* @author: M.Winkler, doubleSlash GmbH
*
* @param key java.lang.Object
* @param newValue java.lang.Object
*/
public void associateAll(Object key, Collection newValues) {
   Iterator iterator = newValues.iterator();
   while (iterator.hasNext()) {
      Object next = iterator.next();
      associate(key, next);
   }
}



/**
* Associates the value to all passed keys.
* <br>Creation date: (27.07.2000 12:51:05)<br>
* @author: M.Winkler, doubleSlash GmbH
*
* @param key java.lang.Object
* @param newValue java.lang.Object
*/
public void associateAll(Collection keys, Object value) {
   Iterator keysIterator = keys.iterator();
   while (keysIterator.hasNext())
    associate(keysIterator.next(), value);

}



/**
* Associates all newValues to all keys found in the passed collctions.
* <br>Creation date: (27.07.2000 12:51:05)<br>
* @author: M.Winkler, doubleSlash GmbH
*
* @param key java.lang.Object
* @param newValue java.lang.Object
*/
public void associateAll(Collection keys, Collection newValues) {

   // retrieve the keys iterator
   Iterator keysIterator = keys.iterator();

   // reuse associate (Object, Collection)
   // since this supports the localizatin, nthis is a performance optimization.
   while (keysIterator.hasNext())
    associateAll(keysIterator.next(), newValues);
}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param setClass java.lang.Class
 * @exception java.lang.InstantiationException Die Beschreibung der Ausnahmebedingung.
 * @exception java.lang.IllegalAccessException Die Beschreibung der Ausnahmebedingung.
 */
public Associator(Class setClass) throws java.lang.InstantiationException, java.lang.IllegalAccessException {

   // check wether this class' enwInstance is acciessible
   Set testSet = (Set) setClass.newInstance();
   setClass_ = setClass;
}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param setClass java.lang.Class
 * @param bidirectional boolean
 * @exception java.lang.InstantiationException Die Beschreibung der Ausnahmebedingung.
 * @exception java.lang.IllegalAccessException Die Beschreibung der Ausnahmebedingung.
 */
public Associator(Class setClass, boolean bidirectional)
   throws java.lang.InstantiationException, java.lang.IllegalAccessException {

   // check wether this class' enwInstance is acciessible
   Set testSet = (Set) setClass.newInstance();
   setClass_ = setClass;
   bidirectional_ = bidirectional;

}





/**
   * <br>Creation date: (27.07.2000 13:13:11)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return java.util.List
   * @param key java.lang.Object
   */
public Set getAssociations(Object key) {
   Set returnValue = pNewSet();
   returnValue = (Set) relationalData_.get(key);
   return returnValue;
}





/**
 * Returns a merge of all associations of all keys in the list.
   * <br>Creation date: (27.07.2000 13:13:11)<br>
   * @author: M.Winkler, doubleSlash GmbH
   *
   * @return java.util.List A list of (valid) keys
   * @param keyList java.util.List A list of values, never null.
   */
public Set getAssociations(List keyList) {
   Set returnValue = pNewSet();
   Iterator keyIterator = keyList.iterator();
   while (keyIterator.hasNext())
      returnValue.addAll((Set) relationalData_.get(keyIterator.next()));
   return returnValue;
}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @return java.lang.Class
 */
public java.lang.Class getSetClass() {
   return setClass_;
}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @return java.util.Set
 */
private Set pNewSet() {
   Set returnValue = null;
   try {
      returnValue = (Set) setClass_.newInstance();
   } catch (Exception e) {
      System.out.println("Exception occurred while creating a new Set instance:" + e.toString());
      e.printStackTrace(System.out);
      returnValue = null;
   }
   return returnValue;
}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param key java.lang.Object
 */
private void pRemove(Object key) {
   Set assocs = (Set) relationalData_.get(key);
   if (assocs != null) {
      assocs.clear();
      relationalData_.remove(key);
   }
}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param newSetClass java.lang.Class
 */
public void setSetClass(java.lang.Class newSetClass) throws InstantiationException, IllegalAccessException {

   // First test if we can use this class to get a new set
   Set newTestSet = (Set) newSetClass.newInstance();

   // check wether we got something useful
   if (newTestSet != null)
      setClass_ = newSetClass;
   else
      throw new InstantiationException("Cannot instantiate " + newSetClass.getName() + ": newInstance() returning null.");

}
}
