package org.jutils.collection;

/**
 * Defines additional functions of a dynamic <code>List</code> in a more abstract way.
 * <p>------------------------------------------------------------------------------<br>
 * Name:     	DynamicList<br>
 * Project:  	jutils.org<br>
 * Copyright:	jutils, see license file<br>
 * Creation date: (05.07.2000 12:10:19)   <br>
 * Version:   <pre>$Id: DynamicList.java,v 1.8 2000/10/05 17:00:07 winkler Exp $</pre> <br>
 * ------------------------------------------------------------------------------<p>
 * @author: M.Winkler, doubleSlash GmbH
 */
public interface DynamicList {
    public static final int RETURN_NULL = 0;
    public static final int RETURN_LAST = 1;
    public static final int RETURN_EMPTY = 2;
    public static final int THROW_EXCEPTION = 3;
    public static final int ADAPT = 4;



    /**
    * The number of elements that this List can take w/o resizing.
    * <br>Creation date: (12.07.2000 14:32:24)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return int
    */
    int capacity();



    /**
    * Removes all elements from the list, when present.
    * <br>Creation date: (26.07.2000 17:40:10)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    */
    void clear();



    /**
    * Clones the dynamic list.
    * <br>Creation date: (27.07.2000 14:31:03)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return org.jutils.collection.DynamicList
    */
    DynamicList cloneList();



    /**
    * Ensures that at least that much elements may be contained by this list.
    * <br>Creation date: (12.07.2000 14:48:16)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return int
    * @param newCapacity int
    */
    int ensureCapacity(int newCapacity);



    /**
    * Returns what happens when a wrong index is given to the getter and setters.
    * Is one of the following:<b>
    * <li>RETURN_NULL null is returned.</li>
    * <li>RETURN_LAST the last returned value will be returned.</li>
    * <li>THROW_EXCEPTION an exception is thrown.</li>
    * <li>ADAPT Negative values are made positive, and too high indexes are modulo'ed to gain a correct index. </li>
    * <br>Creation date: (12.07.2000 14:43:43)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return int
    */
    int getIndexOutOfBoundsBehavior();



    /**
    * Returns true if this list automatically raises capacity when more capacity is needed.
    * <br>Creation date: (12.07.2000 14:33:31)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return boolean
    */
    boolean isAutoResizing();



    /**
    * Sets the default chars that will be used when making a <code>toString ()</code>.
    * @see toString(char,char,char)
    * <br>Creation date: (12.07.2000 14:38:39)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @param openBracket Stringr
    * @param separator String
    * @param closeBracket String
    */
    void setListChars(String openBracket, String separator, String closeBracket);



    /**
    * Returns this list inclusively all elements listed with the provided characters.
    * <br>Creation date: (12.07.2000 14:37:26)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return java.lang.String
    * @param openBracket String
    * @param separator String
    * @param closeBracket String
    */
    String toString(String openBracket, String separator, String closeBracket);



    /**
    * Returns a specific element as a base object. To avoid casting, re-implement a similar method returning
    * the original type of object.
    * <br>Creation date: (12.07.2000 14:34:33)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return java.lang.Object The object at the specific position.
    * @param index int The position from which the element is taken.
    */
    Object get(int index);



    /**
    * Returns the number of Elements stored in this class.
    * <br>Creation date: (12.07.2000 14:31:48)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return int
    */
    int size();
}

