package org.jutils.collection.validators;
import java.util.*;

/**
* This validator checks for values to be (or not to be) in a given set of elements.
* You can pass that set of (il)legal values with a constructor, and specify wether this is an inclusion or en exclusion list.
* If an exclusion list, then the value is considered valid if it is NOT in the given set.
* @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
*/
public class ClassValidator extends AbstractValidator {
   private Class classToCheckAgainst_ = null;
   private boolean acceptSubClasses_ = true;





/**
 * Creates a new ClassValidator.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param templateClass java.lang.Class The class thatb should be tested against.
 * @param acceptSubclasses boolean Defines wether or not subclasses of the specified class are accepted.
 */
public ClassValidator(Class templateClass, boolean acceptSubClasses) {
   if (templateClass == null)
     throw new IllegalArgumentException("argument 'templateClass' is null.");
   classToCheckAgainst_ = templateClass;
   acceptSubClasses_ = acceptSubClasses;
}





/**
 * Startet die Anwendung.
 * @param args ein Array von Befehlszeilenargumenten
 */
public static void main(java.lang.String[] args) {
   System.out.println("Checking ClassValidator ...");
   int errorCount = 0;

   // check a single value

   ClassValidator stringValidator = new ClassValidator(String.class, false);
   System.out.println("Checking with String instance...");
   try {
     stringValidator.checkValid("blabla");
   } catch (InvalidValueException ive) {
     // this is what should NOT happen
     System.out.println("ERROR --- valid value did fail!");
     errorCount++;
   } catch (InvalidTypeException ite) {
     // this is what should NOT happen
     System.out.println("ERROR --- valid value did returned 'wrong type'!");
     errorCount++;
   }

   // check null
   System.out.println("Checking null ...");
   try {
     stringValidator.checkValid(null);
     System.out.println("ERROR --- null value returned ok!");
     errorCount++;
   } catch (InvalidValueException ive) {
     // this is what should happen
   } catch (InvalidTypeException ite) {
     // this is what should NOT happen
     System.out.println("ERROR --- null value returned wrong exception!");
     errorCount++;
   }

   // checking for subclasses
   System.out.println("checking for subclasses, acceptSubclasses enabled ...");
   ClassValidator interfaceValidator = new ClassValidator(Map.class, true);
   try {
     interfaceValidator.checkValid(new HashMap());

   } catch (InvalidValueException ive) {
     // this is what should NOT happen
     System.out.println("ERROR --- valid type did fail value check!");
     errorCount++;

   } catch (InvalidTypeException ite) {

     // this is what should NOT happen
     System.out.println("ERROR --- valid type did fail type check!");
     errorCount++;

   }

   // stats
   System.out.println("--------------------------");
   System.out.println("" + errorCount + " errors detected.");
   System.out.println("--------------------------");
   System.exit(errorCount);
}





/**
 * @see org.jutils.collection.validators.Validator
 */
public void checkValid(java.lang.Object value)
   throws InvalidValueException, InvalidTypeException {

   if (value == null)
     throw new InvalidValueException("value is null");

   boolean valid = false;
   if (acceptSubClasses_)
     //	  valid = value.getClass().isAssignableFrom(classToCheckAgainst_);
     valid = classToCheckAgainst_.isAssignableFrom(value.getClass());
   else
     valid = classToCheckAgainst_.equals(value.getClass());

   // if we are invalid, throw exception.
   if (!valid)
     throw new InvalidTypeException("invalid type");

}





/**
 * Code, der bei der Bereinigung dieses Objekts ausgefhrt wird.
 *
 * Jede von einer Beendigungsmethode bergebene Ausnahmebedingung stoppt die
 * Beendigung. Andernfalls wird er ignoriert.
 */
protected void finalize() throws Throwable {
   // Hier Code einfgen, um den Empfnger zu beenden.
   // Diese Implementierung leitet die Nachricht einfach nur an die Superkomponente weiter. Sie knnen dies ersetzen oder ergnzen.
   super.finalize();
}





/**
 * Gibt eine Zeichenfolge zurck, die den Wert dieses Objekts darstellt.
 * @return eine Zeichenfolgedarstellung des Empfngers
 */
public String toString() {

   StringBuffer returnValue = new StringBuffer(30);
   returnValue.append("VClassValidator checking values for being ");
   if (acceptSubClasses_)
     returnValue.append("casteable to ");
   else
     returnValue.append("of class ");

   returnValue.append(classToCheckAgainst_.getName());

   return returnValue.toString();

}
}
