package org.jutils.collection.validators;

/**
 * This is a straightforward definition of an Object validator. It does nothing but checking if
 * a passed value is "ok".<p>
 * There are two  methods to be implemented by an implementor,
 * but the AbstractValidator already implements one of them, so you only have to implement one. <p>
 * To see a demo of the class and the handling, see the <tt>ClassValidator</tt>'s main method.<p>
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @see org.jutils.collection.validators.AbstractValidator
 * @see org.jutils.collection.validators.ClassValidator
 * @see org.jutils.collection.validators.SetValidator
 */
public interface Validator {



/**
 * This method is the "long" version of the isValid(Object)-Method. Its checking
 * type and value, and if one of them is failing, its throwing an appropriate exception.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @exception org.jutils.collection.validators.InvalidValueException Is thrown if the passed value is out of range (too low, too high, not an element of the possible value set), or simply null.
 * @exception org.jutils.collection.validators.InvalidTypeException Is thrown if the passed value has the wrong type (wrong class or interface).
 */
void checkValid(Object value)
   throws InvalidValueException, InvalidTypeException;





/**
 * Checks wether or not the passed value is valid. Since this is a not very informative method,
 * you also can use checkValid(Object), which is much more informative.<p>
 * As to say, this is a convenience bethod for a straightforward implementor.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @return boolean True if the value is valid.
 * @param value java.lang.Object The value to validate. May be null, if allowed by the implementor.
 */
boolean isValid(Object value);
}
