package org.jutils.init;
import org.jutils.text.StringColumnizer;
import javax.swing.tree.*;
import java.util.*;
import org.jutils.runtime.*;

/**
 * This is somewhat strange - a tree model which shows the structure of a <tt>Properties</tt> instance.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 */
public class PropertiesTreeModel implements TreeModel {
   private String root_ = "root";
   private Map nodes_ = null;





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param properties java.util.Properties
 */
public PropertiesTreeModel(Properties properties) {

   // init
   nodes_ = new Hashtable();

   // partition all properties entries
   Enumeration keys = properties.keys();

   while (keys.hasMoreElements()) {
      String singlePath = keys.nextElement().toString();
      String value = properties.getProperty(singlePath);
      pInsert(singlePath, value);
   }

}





/**
 * @see javax.swing.tree.TreeModel
 */
public void addTreeModelListener(javax.swing.event.TreeModelListener l) {}





/**
 * @see javax.swing.tree.TreeModel
 */
public Object getChild(Object parent, int index) {

   List children = (List) nodes_.get(parent);
   Object child = children.get(index);
   return child;
}





/**
 * @see javax.swing.tree.TreeModel
 */
public int getChildCount(Object parent) {
   int returnValue = 0;
   List nodeList = (List) nodes_.get(parent);
   if (nodeList != null)
      returnValue = nodeList.size();
   return returnValue;

}





/**
 * @see javax.swing.tree.TreeModel
 */
public int getIndexOfChild(Object parent, Object child) {
   List children = (List) nodes_.get(parent);
   int index = children.indexOf(child);
   return index;
}





/**
 * @see javax.swing.tree.TreeModel
 */
public Object getRoot() {
   return root_;
}





/**
 * @see javax.swing.tree.TreeModel
 */
public boolean isLeaf(Object node) {
   boolean returnValue = true;
   List nodesList = (List) nodes_.get(node);
   if (nodesList != null) {
      returnValue = nodesList.size() < 1;
   } else
      returnValue = true;
   return returnValue;
}





/**
 * Recursively inserts this childKey into the tree.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param parentKey java.lang.String
 * @param childKey java.lang.String
 */
private void pInsert(String[] path, String value) {
   // insert into root
   List rootList = (List) nodes_.get(root_);
   if (rootList == null) {
      rootList = Recycler.getVector();
      nodes_.put(root_, rootList);
   }
   rootList.add(path[0]);

   // iterate through path

   if (path.length > 1) {
      for (int i = 0; i < path.length - 1; i++) {
         String node = path[i];
         String child = path[i + 1];
         List children = (List) nodes_.get(node);
         if (children == null) {
            children = Recycler.getVector();
            nodes_.put(node, children);
         }
         if (!children.contains(child))
            children.add(child);
      }
   }

   // insert value as leaf
   String lastNode = path[path.length - 1];
   List leafs = (List) nodes_.get(lastNode);
   if (leafs == null) {
      leafs = Recycler.getVector();
      nodes_.put(lastNode, leafs);
   }
   if (!leafs.contains(value))
      leafs.add(value);

}





/**
 * Recursively inserts this childKey into the parent.
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param parentKey java.lang.String
 * @param childKey java.lang.String
 */
private void pInsert(String path, String value) {
   StringColumnizer columnizer = new StringColumnizer(path, ".");
   String[] splittedPath = columnizer.toArray(null);
   pInsert(splittedPath, value);

}





/**
 * @author M.Winkler, doubleslash NetBusiness GmbH, Friedrichshafen
 * @param parentKey java.lang.String
 * @param childKey java.lang.String
 */
private void pInsertChild(String parentKey, String childKey) {}





/**
 * @see javax.swing.tree.TreeModel
 */
public void removeTreeModelListener(javax.swing.event.TreeModelListener l) {}





/**
 * @see javax.swing.tree.TreeModel
 */
public void valueForPathChanged(TreePath path, Object newValue) {}
}
