package org.jutils.jdbc;

import java.util.*;
import java.text.*;
/**
 * Some tools that are made for convenience and are especially
 * valuable in the context of Jdbc.
 * CAUTION! This is a hack! This class will be splitted into
 * multiple classes later on; therefore this class is marked
 * deprecated from the beginning!
 * @deprecated
 */
public class JdbcTools {
    private static DateFormat lastUsedDateFormat_ = null;



    /************************************************************
          Ermittelt das aktuelle Datum mit gewnschtem Look and Feel
          <br><b>Beispiel:</b><br>
        <code>Utils.getCurrentDate(DateFormat.MEDIUM, Locale.GERMAN);</code><br>
        -> returns 09.02.1999
          @param lookAndFeel transforms for example jdbcDateStr="1998-02-09" to:<br>
         java.text.DateFormat.SHORT -> 09.02.98<br>
         java.text.DateFormat.MEDIUM-> 09.02.1998<br>
         java.text.DateFormat.LONG  -> 9. Februar 1998<br>
         java.text.DateFormat.FULL  -> Montag, 9. Februar 1998<p>
          @param country is for example<br>
         java.util.Locale.GERMAN -> 09.02.98<br>
         java.util.Locale.US -> 2/9/98  (or February 9, 1998)<p>
          @return
         Das aktuelle Datum, schn formatiert fr das jeweilige Land
          ************************************************************/
        public static final String getCurrentDate(final int lookAndFeel, final java.util.Locale country) {
            String returnValue = null;
            java.util.Date currentDate = new java.util.Date();
            java.text.DateFormat df = java.text.DateFormat.getDateInstance(lookAndFeel, country);
            returnValue = df.format(currentDate);
            return returnValue;
        }



    /************************************************************
          Ermittelt das aktuelle Datum plus die aktuelle Uhrzeit mit gewnschtem Look and Feel
          <br><b>Beispiel:</b><br>
        <code>Utils.getCurrentDateTime(DateFormat.MEDIUM, DateFormat.MEDIUM, Locale.GERMAN);</code><br>
        -> return "08.03.1999 15:28:19"
          @param lookAndFeelDate java.text.DateFormat.SHORT<br>
         java.text.DateFormat.MEDIUM<br>
         java.text.DateFormat.LONG<br>
         java.text.DateFormat.FULL
          @param lookAndFeelTime java.text.DateFormat.SHORT<br>
         java.text.DateFormat.MEDIUM<br>
         java.text.DateFormat.LONG<br>
         java.text.DateFormat.FULL
          @param country         java.util.Locale.GERMAN<br>
         java.util.Locale.US
          @return
         Das aktuelle Datum/Uhrzeit, schn formatiert fr das jeweilige Land
          ************************************************************/
        public static final String getCurrentDateTime(final int lookAndFeelDate, final int lookAndFeelTime, final java.util.Locale country) {
            String returnValue = null;
            java.util.Date currentDate = new java.util.Date();
            java.text.DateFormat df = java.text.DateFormat.getDateTimeInstance(lookAndFeelDate, lookAndFeelTime, country);
            returnValue = df.format(currentDate);
            return returnValue;
        }



    /************************************************************
          Ermittelt das aktuelle Datum plus die aktuelle Uhrzeit im JDBC Format<br>
          <b>BUG:</b> Only returns Date but no Time
          @return
         Das aktuelle Datum/Uhrzeit im JDBC Format
          ************************************************************/
        public static final String getCurrentJDBCDateTime() {
            String returnValue = null;
            java.util.Date currentDate = new java.util.Date();
            java.sql.Date jdbcDate = new java.sql.Date(currentDate.getTime());
            returnValue = jdbcDate.toString();
            return returnValue;
        }



    /************************************************************
          Ermittelt die aktuelle Uhrzeit mit gewnschtem Look and Feel
          <br><b>Beispiel:</b><br>
        <code>Utils.getCurrentTime(DateFormat.MEDIUM, Locale.GERMAN);</code>
          @param lookAndFeel java.text.DateFormat.SHORT<br>
         java.text.DateFormat.MEDIUM<br>
         java.text.DateFormat.LONG<br>
         java.text.DateFormat.FULL
          @param country     java.util.Locale.GERMAN<br>
         java.util.Locale.US
          @return
         Die aktuelle Uhrzeit, schn formatiert fr das jeweilige Land
          ************************************************************/
        public static final String getCurrentTime(final int lookAndFeel, final java.util.Locale country) {
            String returnValue = null;
            java.util.Date currentDate = new java.util.Date();
            java.text.DateFormat df = java.text.DateFormat.getTimeInstance(lookAndFeel, country);
            returnValue = df.format(currentDate);
            return returnValue;
        }



    /**
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.text.DateFormat
         */
        public static java.text.DateFormat getLastUsedDateFormat() {
            return lastUsedDateFormat_;
        }



    /**
     * Tests the functions of this helper class.
     * Creation date: (06.11.2000 15:57:25)
     * @param args java.lang.String[]
     */
    public static void main(String[] args) throws ParseException {
        int errorCount= 0;
    
        // Test parsing from/to java.util.Date
    
        System.out.println("------------------------------");
        System.out.println("Test parsing ...");
        System.out.println("------------------------------");
        Date now= new Date();
        DateFormat df= DateFormat.getDateTimeInstance();
        String nowString= df.format(now);
    
        System.out.println("nowString is " + nowString);
        String jdbcDate= toJDBCDate(now);
        System.out.println("jdbcDate is " + jdbcDate);
        Date reconvertedDate= toDateField(jdbcDate);
        long diff= Math.abs(now.getTime() - reconvertedDate.getTime());
        if (diff != 0) {
            errorCount++;
            long minMillis= 1000 * 60;
            long hourMillis= minMillis * 60;
    
            long dayMillis= hourMillis * 24;
    
            long diffDays= (long) ((float) diff / dayMillis);
            long diffRest= diff - (diffDays * dayMillis);
            long diffHours= diffRest / hourMillis;
            diffRest= diffRest - (diffHours * hourMillis);
            long diffMinutes= diffRest / minMillis;
            diffRest= diffRest - (diffMinutes * minMillis);
            long diffSecs= diffRest / 1000;
            diffRest= diffRest - (diffSecs * 1000);
            System.out.println(
                "reconverted value does not match with original value!\n Diff is "
                    + diff
                    + " ticks, which equals "
                    + diffDays
                    + " d "
                    + diffHours
                    + " h "
                    + diffMinutes
                    + " m "
                    + diffSecs
                    + " s and "
                    + diffRest
                    + " millis."); 
        }
    
        System.out.println("==============================");
        System.out.println("" + errorCount + " errors detected.");
        System.out.println("------------------------------");
    
    }



    /**
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @param newLastUsedDateFormat java.text.DateFormat
         */
        public static void setLastUsedDateFormat(java.text.DateFormat newLastUsedDateFormat) {
            lastUsedDateFormat_ = newLastUsedDateFormat;
        }



    /**
         * @author Michael Winkler, doubleSlash GmbH
         *
         * @return java.util.Date
         * @param jdbcDateString java.lang.String
         */
        public static Date toDateField(String jdbcDateString) throws ParseException {
            Date returnValue = null;
            String dateFormat = toLocalDateFormat(jdbcDateString);
            returnValue = lastUsedDateFormat_.parse(dateFormat);
            return returnValue;
        }



    /************************************************************
          Parses a German date string and returns the standard JDBC date format
          <br><b>Beispiel:</b><br>
        <code>Utils.toJDBCDate("09.02.1998");</code><br>
        -> returns 1998-02-09
          @param localDateStr is a typical german date string<br>
         for example: 24.12.1998, 4.1.98 or 9. Februar 1998
          @return
         date in JDBC format, for example 1998-02-09
          ************************************************************/
        public static final String toJDBCDate(String localDateStr) throws java.text.ParseException {
            String returnValue = null;
            java.util.Locale locale = java.util.Locale.getDefault();
            returnValue = toJDBCDate(locale, localDateStr);
            return returnValue;
        }



    /**
         * Converts the passed Date instance into a String looking like 'yyyy-mm-dd', which is the standard jdbc date format.
         **/
    public static final String toJDBCDate(java.util.Date date) {
        StringBuffer returnValue= new StringBuffer();
    
        // compose and append year string
        Calendar c= Calendar.getInstance();
        c.setTime(date);
        returnValue.append(c.get(c.YEAR));
        returnValue.append('-');
    
        // compose and append month string
        
        // MW 6.11.00
        // c.get(c.MONTH) seems to return one less counter:
        // - if you want 1 for january, you will get 0.
        // So we have to raise this result by one.
        String monthString= String.valueOf(c.get(c.MONTH) + 1);
        if (monthString.length() < 2)
            monthString= '0' + monthString;
        returnValue.append(monthString);
        returnValue.append('-');
    
        // compose and update day string
        String dayString= String.valueOf(c.get(c.DAY_OF_MONTH));
        if (dayString.length() < 2)
            dayString= '0' + dayString;
        returnValue.append(dayString);
    
        // thats it.
        return returnValue.toString();
    }



    /************************************************************
          Parses an international date string and returns
          the standard JDBC date format
          <br><b>Beispiel:</b><br>
        <code>Utils.toJDBCDate("09.02.1998");</code><br>
        -> returns 1998-02-09
          @param localDateStr is a typical german date string<br>
         for example: 24.12.1998, 4.1.98 or 9. Februar 1998
          @param country is for example<br>
         java.util.Locale.GERMAN -> 09.02.98<br>
         java.util.Locale.US -> 2/9/98  (or February 9, 1998)
          @return
         date in JDBC format, for example 1998-02-09
          ************************************************************/
        public static final String toJDBCDate(java.util.Locale country, String localDateStr) throws java.text.ParseException {
            String returnValue = null;
            java.text.DateFormat df = java.text.DateFormat.getDateInstance(java.text.DateFormat.SHORT, country);
            java.util.Date d_de;
            try {
                d_de = df.parse(localDateStr); // "27.12.1998"
            }
            catch (java.text.ParseException e) {
                try {
                    df = java.text.DateFormat.getDateInstance(java.text.DateFormat.MEDIUM, country);
                    d_de = df.parse(localDateStr); // "27.12.1998"
                }
                catch (java.text.ParseException e4) {
                    // Log.trace("TRYING parse again: " + e4.toString());
                    try {
                        df = java.text.DateFormat.getDateInstance(java.text.DateFormat.LONG, country);
                        d_de = df.parse(localDateStr); // "9. Februar 1998"
                    }
                    catch (java.text.ParseException e2) {
                        // Log.trace("TRYING again ...: " + e2.toString());
                        df = java.text.DateFormat.getDateInstance(java.text.DateFormat.FULL, country);
                        d_de = df.parse(localDateStr); // "Montag, 9. Februar 1998"
    
                    }
                }
            }
            java.sql.Date sqlDate = new java.sql.Date(d_de.getTime());
            if (sqlDate != null)
                returnValue = sqlDate.toString();
            else
                throw new java.text.ParseException("Could not convert local date String '" + localDateStr + "' to Jdbc date string!", 0);
            return returnValue;
        }



    /************************************************************
          Converts the JDBC date format to your country look and feel
          <br><b>Beispiel:</b><br>
        <code>Utils.toLocalDateFormat(java.text.DateFormat.SHORT, java.util.Locale.GERMAN, "1998-02-09")</code><br>
        -> returns 09.02.98<br>
        <code>Utils.toLocalDateFormat(java.text.DateFormat.LONG, java.util.Locale.US, "1998-02-09")</code><br>
        -> returns February 9, 1998
          @param lookAndFeel transforms for example jdbcDateStr="1998-02-09" to:<br>
         java.text.DateFormat.SHORT -> 09.02.98<br>
         java.text.DateFormat.MEDIUM-> 09.02.1998<br>
         java.text.DateFormat.LONG  -> 9. Februar 1998<br>
         java.text.DateFormat.FULL  -> Montag, 9. Februar 1998
          @param country is for example<br>
         java.util.Locale.GERMAN -> 09.02.98<br>
         java.util.Locale.US -> 2/9/98  (or February 9, 1998)
          @param jdbcDateStr is the java.sql.ResultSet.getDate() format<br>
         for example: 1998-02-09<br>
         this is the JDBC internal format
          @return
         your desired date, nicely formatted to you country look and feel
          ************************************************************/
        public static final String toLocalDateFormat(final int lookAndFeel, final java.util.Locale country, String jdbcDateStr) {
            // String JDBC_Date = new String("1998-02-09 00:00:00.0");
            String returnValue = null;
            java.text.DateFormat df = java.text.DateFormat.getDateInstance(lookAndFeel, country);
            java.sql.Date sqlDate = java.sql.Date.valueOf(jdbcDateStr);
            returnValue = df.format(sqlDate);
            return returnValue;
        }



    /************************************************************
          Converts the JDBC date format to a localized date string, using default country and look and feel
    
         java.text.DateFormat.SHORT -> 09.02.98<br>
         java.text.DateFormat.MEDIUM-> 09.02.1998<br>
         java.text.DateFormat.LONG  -> 9. Februar 1998<br>
         java.text.DateFormat.FULL  -> Montag, 9. Februar 1998
          @param country is for example<br>
         java.util.Locale.GERMAN -> 09.02.98<br>
         java.util.Locale.US -> 2/9/98  (or February 9, 1998)
         @param jdbcDateStr is the java.sql.ResultSet.getDate() format<br>
         for example: 1998-02-09<br>
         this is the JDBC internal format
          @return
         your desired date, nicely formatted to you country look and feel
          ************************************************************/
    public static final String toLocalDateFormat(String jdbcDateStr) {
        // String JDBC_Date = new String("1998-02-09 00:00:00.0");
        String returnValue= null;
        if (lastUsedDateFormat_ == null) {
            int lookAndFeel= java.text.DateFormat.DEFAULT;
            Locale locale= Locale.getDefault();
            lastUsedDateFormat_= DateFormat.getDateInstance(lookAndFeel, locale);
        }
        returnValue= toLocalDateFormat(lastUsedDateFormat_, jdbcDateStr);
        return returnValue;
    }



    /************************************************************
          Converts the JDBC date format to your country look and feel
          <br><b>Beispiel:</b><br>
        <code>Utils.toLocalDateFormat(java.text.DateFormat.SHORT, java.util.Locale.GERMAN, "1998-02-09")</code><br>
        -> returns 09.02.98<br>
        <code>Utils.toLocalDateFormat(java.text.DateFormat.LONG, java.util.Locale.US, "1998-02-09")</code><br>
        -> returns February 9, 1998
          @param lookAndFeel transforms for example jdbcDateStr="1998-02-09" to:<br>
         java.text.DateFormat.SHORT -> 09.02.98<br>
         java.text.DateFormat.MEDIUM-> 09.02.1998<br>
         java.text.DateFormat.LONG  -> 9. Februar 1998<br>
         java.text.DateFormat.FULL  -> Montag, 9. Februar 1998
          @param country is for example<br>
         java.util.Locale.GERMAN -> 09.02.98<br>
         java.util.Locale.US -> 2/9/98  (or February 9, 1998)
          @param jdbcDateStr is the java.sql.ResultSet.getDate() format<br>
         for example: 1998-02-09<br>
         this is the JDBC internal format
          @return
         your desired date, nicely formatted to you country look and feel
          ************************************************************/
        public static final String toLocalDateFormat(DateFormat df, String jdbcDateStr) {
            // String JDBC_Date = new String("1998-02-09 00:00:00.0");
            String returnValue = null;
            java.sql.Date sqlDate = java.sql.Date.valueOf(jdbcDateStr);
            returnValue = df.format(sqlDate);
            return returnValue;
        }



    /**
         * JdbcTools constructor comment.
         */
        private JdbcTools() {
            super();
        }
}

